/*
 * Renderer 1. The MIT License.
 * Copyright (c) 2022 rlkraft@pnw.edu
 * See LICENSE for details.
*/

import renderer.scene.*;
import renderer.models_L.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.io.File;
import java.awt.Color;

/**
   This program creates an animation by repeatedly
   saving the framebuffer's contents to a file.
   <p>
   You can see the animation by quickly sequencing
   through the frame images.
*/
public class OfflineMovie
{
   private static final String assets = "../assets/";

   public static void main(String[] args)
   {
      // Create the Scene object that we shall render.
      final Scene scene = new Scene();

      // Create several Model objects.
      final Model m1 = new ObjSimpleModel(new File(assets + "apple.obj"));
      final Model m2 = new ObjSimpleModel(new File(assets + "cow.obj"));
      final Model m3 = new ObjSimpleModel(new File(assets + "cessna.obj"));
      final Model m4 = new ObjSimpleModel(new File(assets + "galleon.obj"));
      final Model m5 = new ObjSimpleModel(new File(assets + "teapot.obj"));
      final Model m6 = new ObjSimpleModel(new File(
                          assets + "small_rhombicosidodecahedron.obj"));
      final Model m7 = new ObjSimpleModel(new File(
                          assets + "great_rhombicosidodecahedron.obj"));

      // Add some models to the Scene.
      scene.addPosition(new Position(m1, "p0"),  // apple
                        new Position(m3, "p1"),  // cessna
                        new Position(m4, "p2"),  // galleon
                        new Position(m2, "p3")); // cow

      // Push the models away from where the camera is.
      for (final Position p : scene.positionList)
      {
         p.translation(0.0,  0.0, -3.0);
      }

      // Log information to standard output.
      System.out.println( scene.toString() );

      // Create a framebuffer to render our Scene into.
      final int width  = 1024;
      final int height = 1024;
      final FrameBuffer fb = new FrameBuffer(width, height, Color.darkGray);

      Rasterize.doClipping = false; // turn off clipping

      // Render our scene into the framebuffer.
      Pipeline.render(scene, fb);
      // Save the resulting image in a file.
      fb.dumpFB2File( String.format("PPM_OfflineModels_Frame%03d.ppm", 0) );

      for (int j = 1; j <= 100; ++j)
      {
         // Update the scene.
         Vector t = scene.getPosition(0).getTranslation();
         scene.getPosition(0).translation(t.x + 0.3,  // move right
                                          t.y + 0.1,  // move up
                                          t.z - 0.1); // move back

         t = scene.getPosition(1).getTranslation();
         scene.getPosition(1).translation(t.x - 0.3,  // move left
                                          t.y - 0.1,  // move down
                                          t.z - 0.1); // move back

         t = scene.getPosition(2).getTranslation();
         scene.getPosition(2).translation(t.x + 0.01,  // move right
                                          t.y,
                                          t.z);

         // Render again.
         fb.clearFB(); // What happens if we don't clear the framebuffer?
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_OfflineModels_Frame%03d.ppm", j));
      }
   }
}
