/*
 * Renderer 1. The MIT License.
 * Copyright (c) 2022 rlkraft@pnw.edu
 * See LICENSE for details.
*/

import renderer.scene.*;
import renderer.models_L.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;

/**
   http://merganser.math.gvsu.edu/david/psseminar/index.html#jiggity
*/
public class ThreeDimensionalScene_R1
{
   public static void main(String[] args)
   {
      // Create the Scene object that we shall render.
      final Scene scene = new Scene();

      // Create several Model objects.
      final Model m0 = new Cube();
      final Model m1 = new Tetrahedron();
      final Model m2 = new Octahedron();
      final Model m3 = new Dodecahedron();
      final Model m4 = new Icosahedron();
      final Model m5 = new PanelXZ(-7, 7, -7, 7);

      // Add the models to the Scene.
      scene.addPosition(new Position(m0, "p0"),  // Cube
                        new Position(m1, "p1"),  // Tetrahedron
                        new Position(m2, "p2"),  // Octahedron
                        new Position(m3, "p3"),  // Dodecahedron
                        new Position(m4, "p4"),  // Icosahedron
                        new Position(m5, "p5")); // Panel

      // Place each model in front of the camera.
      scene.getPosition(5).translation( 0, -7, -14);  // Panel
      scene.getPosition(0).translation(-4,  0, -18);  // Cube
      scene.getPosition(1).translation( 0, -2, -14);  // Tetrahedron
      scene.getPosition(2).translation( 4, -3, -10);  // Octahedron
      scene.getPosition(3).translation(-4, -3, -10);  // Dodecahedron
      scene.getPosition(4).translation( 4,  0, -18);  // Icosahedron

      // Create a FrameBuffer to render our Scene into.
      final int width  = 800;
      final int height = 800;
      final FrameBuffer fb = new FrameBuffer(width, height, Color.darkGray);

      // Render.
      scene.debug = true;
      Pipeline.render(scene, fb);

      // Save the framebuffer's contents in an image file.
      fb.dumpFB2File("ThreeDimensionalScene_R1.ppm");
   }
}
